# -*- coding: utf-8 -*-


from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('django_nyt', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Settings', fields=[
                ('id', models.AutoField(
                    auto_created=True, verbose_name='ID', serialize=False, primary_key=True)), ('user', models.ForeignKey(
                    to_field='id', to=settings.AUTH_USER_MODEL)), ('interval', models.SmallIntegerField(
                    choices=[
                        (0, 'instantly'), (1380, 'daily'), (9660, 'weekly')], default=0, verbose_name='interval')), ], options={
                'verbose_name': 'settings', 'db_table': 'nyt_settings', 'verbose_name_plural': 'settings', }, bases=(
                models.Model,), ), migrations.CreateModel(
            name='Notification', fields=[
                ('id', models.AutoField(
                    auto_created=True, verbose_name='ID', serialize=False, primary_key=True)), ('message', models.TextField()), ('url', models.CharField(
                    verbose_name='link for notification', null=True, max_length=200, blank=True)), ('is_viewed', models.BooleanField(
                    default=False)), ('is_emailed', models.BooleanField(
                    default=False)), ('created', models.DateTimeField(
                    auto_now_add=True)), ('occurrences', models.PositiveIntegerField(
                    help_text='If the same notification was fired multiple times with no intermediate notifications', default=1, verbose_name='occurrences')), ], options={
                'verbose_name': 'notification', 'db_table': 'nyt_notification', 'ordering': (
                    '-id',), 'verbose_name_plural': 'notifications', }, bases=(
                models.Model,), ), ]



from . import _disable_notifications

from django.db.models import Model
from django.utils.translation import ugettext as _
from . import models


def notify(message, key, target_object=None, url=None, filter_exclude={}, recipient_users=None):
    """
    Notify subscribing users of a new event. Key can be any kind of string,
    just make sure to reuse it where applicable! Object_id is some identifier
    of an object, for instance if a user subscribes to a specific comment thread,
    you could write:

    notify("there was a response to your comment", "comment_response",
           target_object=PostersObject,
           url=reverse('comments:view', args=(PostersObject.id,)))

    The below example notifies everyone subscribing to the "new_comments" key
    with the message "New comment posted".

    notify("New comment posted", "new_comments")

    filter_exclude: a dictionary to exclude special elements of subscriptions
    in the queryset, for instance filter_exclude={''}
    
    :param: recipient_users: A possible iterable of users that should be notified
                             instead of notifying all subscribers of the event.
                             Notice that users still have to be actually subscribed
                             to the event key!
    """

    if _disable_notifications:
        return 0

    if target_object:
        if not isinstance(target_object, Model):
            raise TypeError(
                _("You supplied a target_object that's not an instance of a django Model."))
        object_id = target_object.id
    else:
        object_id = None

    objects = models.Notification.create_notifications(
        key,
        object_id=object_id,
        message=message,
        url=url,
        filter_exclude=filter_exclude,
        recipient_users=recipient_users,
    )
    return len(objects)


def subscribe(settings, key, content_type=None, object_id=None, **kwargs):
    """
    Creates a new subscription to a given key. If the key does not exist
    as a NotificationType, it will be created
    
    If a subscription already exist, it is returned
    
    :param: settings: A models.Settings instance (user + interval specification)
    :param: key: The unique key that the Settings should subscribe to
    :param: content_type: If notifications are regarding a specific ContentType, it should be set
    :param: object_id: If the notifications should only regard a specific object_id
    :param: **kwargs: Additional models.Subscription field values
    """
    notification_type = models.NotificationType.get_by_key(key, content_type=content_type)
    
    models.Subscription.objects.filter(
        settings=settings,
        notification_type=notification_type,
        object_id=object_id,
        **kwargs
    )
    return models.Subscription.objects.create(
        settings=settings,
        notification_type=notification_type,
        object_id=object_id,
        **kwargs
    )


# -*- coding: utf-8 -*-

from django_nyt.decorators import json_view, login_required_ajax
from django_nyt import models
from django.contrib.auth.decorators import login_required
from django.shortcuts import redirect, get_object_or_404
from django.utils.translation import ugettext as _


@login_required_ajax
@json_view
def get_notifications(
        request,
        latest_id=None,
        is_viewed=False,
        max_results=10):

    notifications = models.Notification.objects.filter(
        subscription__settings__user=request.user,)

    if not is_viewed is None:
        notifications = notifications.filter(is_viewed=is_viewed)

    if not latest_id is None:
        notifications = notifications.filter(id__gt=latest_id)

    notifications = notifications.order_by('-id')
    notifications = notifications.prefetch_related(
        'subscription',
        'subscription__notification_type')

    from django.contrib.humanize.templatetags.humanize import naturaltime

    return {'success': True,
            'total_count': notifications.count(),
            'objects': [{'pk': n.pk,
                         'message': n.message,
                         'url': n.url,
                         'occurrences': n.occurrences,
                         'occurrences_msg': _('%d times') % n.occurrences,
                         'type': n.subscription.notification_type.key,
                         'since': naturaltime(n.created)} for n in notifications[:max_results]]}


@login_required
def goto(request, notification_id=None):
    referer = request.META.get('HTTP_REFERER', '')
    if not notification_id:
        return redirect(referer)
    notification = get_object_or_404(models.Notification,
                                     subscription__settings__user=request.user,
                                     id=notification_id)
    notification.is_viewed = True
    notification.save()
    if not notification.url is None:
        return redirect(notification.url)
    return redirect(referer)


@login_required_ajax
@json_view
def mark_read(request, id_lte, notification_type_id=None, id_gte=None):

    notifications = models.Notification.objects.filter(
        subscription__settings__user=request.user,
        id__lte=id_lte)

    if notification_type_id:
        notifications = notifications.filter(
            notification_type__id=notification_type_id)

    if id_gte:
        notifications = notifications.filter(id__gte=id_gte)

    notifications.update(is_viewed=True)

    return {'success': True}



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'NotificationType'
        db.create_table(
            'nyt_notificationtype',
            (('key',
              self.gf('django.db.models.fields.CharField')(
                  unique=True,
                  max_length=128,
                  primary_key=True)),
                ('label',
                 self.gf('django.db.models.fields.CharField')(
                     max_length=128,
                     null=True,
                     blank=True)),
                ('content_type',
                 self.gf('django.db.models.fields.related.ForeignKey')(
                     to=orm['contenttypes.ContentType'],
                     null=True,
                     blank=True)),
             ))
        db.send_create_signal('django_nyt', ['NotificationType'])

        # Adding model 'Settings'
        db.create_table(
            'nyt_settings', (('id', self.gf('django.db.models.fields.AutoField')(
                primary_key=True)), ('user', self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['auth.User'])), ('interval', self.gf('django.db.models.fields.SmallIntegerField')(
                default=0)), ))
        db.send_create_signal('django_nyt', ['Settings'])

        # Adding model 'Subscription'
        db.create_table(
            'nyt_subscription', (('id', self.gf('django.db.models.fields.AutoField')(
                primary_key=True)), ('settings', self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['django_nyt.Settings'])), ('notification_type', self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['django_nyt.NotificationType'])), ('object_id', self.gf('django.db.models.fields.CharField')(
                max_length=64, null=True, blank=True)), ('send_emails', self.gf('django.db.models.fields.BooleanField')(
                default=True)), ('latest', self.gf('django.db.models.fields.related.ForeignKey')(
                blank=True, related_name='latest_for', null=True, to=orm['django_nyt.Notification'])), ))
        db.send_create_signal('django_nyt', ['Subscription'])

        # Adding model 'Notification'
        db.create_table(
            'nyt_notification',
            (('id',
              self.gf('django.db.models.fields.AutoField')(
                  primary_key=True)),
                ('subscription',
                 self.gf('django.db.models.fields.related.ForeignKey')(
                     to=orm['django_nyt.Subscription'],
                     null=True,
                     on_delete=models.SET_NULL,
                     blank=True)),
                ('message',
                 self.gf('django.db.models.fields.TextField')()),
                ('url',
                 self.gf('django.db.models.fields.CharField')(
                     max_length=200,
                     null=True,
                     blank=True)),
                ('is_viewed',
                 self.gf('django.db.models.fields.BooleanField')(
                     default=False)),
                ('is_emailed',
                 self.gf('django.db.models.fields.BooleanField')(
                     default=False)),
                ('created',
                 self.gf('django.db.models.fields.DateTimeField')(
                     auto_now_add=True,
                     blank=True)),
                ('occurrences',
                 self.gf('django.db.models.fields.PositiveIntegerField')(
                     default=1)),
             ))
        db.send_create_signal('django_nyt', ['Notification'])

    def backwards(self, orm):
        # Deleting model 'NotificationType'
        db.delete_table('nyt_notificationtype')

        # Deleting model 'Settings'
        db.delete_table('nyt_settings')

        # Deleting model 'Subscription'
        db.delete_table('nyt_subscription')

        # Deleting model 'Notification'
        db.delete_table('nyt_notification')

    models = {
        'auth.group': {
            'Meta': {
                'object_name': 'Group'}, 'id': (
                'django.db.models.fields.AutoField', [], {
                    'primary_key': 'True'}), 'name': (
                'django.db.models.fields.CharField', [], {
                    'unique': 'True', 'max_length': '80'}), 'permissions': (
                'django.db.models.fields.related.ManyToManyField', [], {
                    'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})}, 'auth.permission': {
            'Meta': {
                'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'}, 'codename': (
                'django.db.models.fields.CharField', [], {
                    'max_length': '100'}), 'content_type': (
                'django.db.models.fields.related.ForeignKey', [], {
                    'to': "orm['contenttypes.ContentType']"}), 'id': (
                'django.db.models.fields.AutoField', [], {
                    'primary_key': 'True'}), 'name': (
                'django.db.models.fields.CharField', [], {
                    'max_length': '50'})}, 'auth.user': {
            'Meta': {
                'object_name': 'User'}, 'date_joined': (
                'django.db.models.fields.DateTimeField', [], {
                    'default': 'datetime.datetime.now'}), 'email': (
                'django.db.models.fields.EmailField', [], {
                    'max_length': '75', 'blank': 'True'}), 'first_name': (
                'django.db.models.fields.CharField', [], {
                    'max_length': '30', 'blank': 'True'}), 'groups': (
                'django.db.models.fields.related.ManyToManyField', [], {
                    'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}), 'id': (
                'django.db.models.fields.AutoField', [], {
                    'primary_key': 'True'}), 'is_active': (
                'django.db.models.fields.BooleanField', [], {
                    'default': 'True'}), 'is_staff': (
                'django.db.models.fields.BooleanField', [], {
                    'default': 'False'}), 'is_superuser': (
                'django.db.models.fields.BooleanField', [], {
                    'default': 'False'}), 'last_login': (
                'django.db.models.fields.DateTimeField', [], {
                    'default': 'datetime.datetime.now'}), 'last_name': (
                'django.db.models.fields.CharField', [], {
                    'max_length': '30', 'blank': 'True'}), 'password': (
                'django.db.models.fields.CharField', [], {
                    'max_length': '128'}), 'user_permissions': (
                'django.db.models.fields.related.ManyToManyField', [], {
                    'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}), 'username': (
                'django.db.models.fields.CharField', [], {
                    'unique': 'True', 'max_length': '30'})}, 'contenttypes.contenttype': {
            'Meta': {
                'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"}, 'app_label': (
                'django.db.models.fields.CharField', [], {
                    'max_length': '100'}), 'id': (
                'django.db.models.fields.AutoField', [], {
                    'primary_key': 'True'}), 'model': (
                'django.db.models.fields.CharField', [], {
                    'max_length': '100'}), 'name': (
                'django.db.models.fields.CharField', [], {
                    'max_length': '100'})}, 'django_nyt.notification': {
            'Meta': {
                'ordering': "('-id',)", 'object_name': 'Notification', 'db_table': "'nyt_notification'"}, 'created': (
                'django.db.models.fields.DateTimeField', [], {
                    'auto_now_add': 'True', 'blank': 'True'}), 'id': (
                'django.db.models.fields.AutoField', [], {
                    'primary_key': 'True'}), 'is_emailed': (
                'django.db.models.fields.BooleanField', [], {
                    'default': 'False'}), 'is_viewed': (
                'django.db.models.fields.BooleanField', [], {
                    'default': 'False'}), 'message': (
                'django.db.models.fields.TextField', [], {}), 'occurrences': (
                'django.db.models.fields.PositiveIntegerField', [], {
                    'default': '1'}), 'subscription': (
                'django.db.models.fields.related.ForeignKey', [], {
                    'to': "orm['django_nyt.Subscription']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}), 'url': (
                'django.db.models.fields.CharField', [], {
                    'max_length': '200', 'null': 'True', 'blank': 'True'})}, 'django_nyt.notificationtype': {
            'Meta': {
                'object_name': 'NotificationType', 'db_table': "'nyt_notificationtype'"}, 'content_type': (
                'django.db.models.fields.related.ForeignKey', [], {
                    'to': "orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}), 'key': (
                'django.db.models.fields.CharField', [], {
                    'unique': 'True', 'max_length': '128', 'primary_key': 'True'}), 'label': (
                'django.db.models.fields.CharField', [], {
                    'max_length': '128', 'null': 'True', 'blank': 'True'})}, 'django_nyt.settings': {
            'Meta': {
                'object_name': 'Settings', 'db_table': "'nyt_settings'"}, 'id': (
                'django.db.models.fields.AutoField', [], {
                    'primary_key': 'True'}), 'interval': (
                'django.db.models.fields.SmallIntegerField', [], {
                    'default': '0'}), 'user': (
                'django.db.models.fields.related.ForeignKey', [], {
                    'to': "orm['auth.User']"})}, 'django_nyt.subscription': {
            'Meta': {
                'object_name': 'Subscription', 'db_table': "'nyt_subscription'"}, 'id': (
                'django.db.models.fields.AutoField', [], {
                    'primary_key': 'True'}), 'latest': (
                'django.db.models.fields.related.ForeignKey', [], {
                    'blank': 'True', 'related_name': "'latest_for'", 'null': 'True', 'to': "orm['django_nyt.Notification']"}), 'notification_type': (
                'django.db.models.fields.related.ForeignKey', [], {
                    'to': "orm['django_nyt.NotificationType']"}), 'object_id': (
                'django.db.models.fields.CharField', [], {
                    'max_length': '64', 'null': 'True', 'blank': 'True'}), 'send_emails': (
                'django.db.models.fields.BooleanField', [], {
                    'default': 'True'}), 'settings': (
                'django.db.models.fields.related.ForeignKey', [], {
                    'to': "orm['django_nyt.Settings']"})}}

    complete_apps = ['django_nyt']



# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url

urlpatterns = patterns(
    '', url('^json/get/$', 'django_nyt.views.get_notifications',
            name='json_get'), url('^json/get/(?P<latest_id>\d+)/$',
                                  'django_nyt.views.get_notifications', name='json_get'), url(
        '^json/mark-read/$', 'django_nyt.views.mark_read',
        name='json_mark_read_base'), url('^json/mark-read/(\d+)/$',
                                         'django_nyt.views.mark_read', name='json_mark_read'), url(
        '^json/mark-read/(?P<id_lte>\d+)/(?P<id_gte>\d+)/$',
        'django_nyt.views.mark_read', name='json_mark_read'), url(
        '^goto/(?P<notification_id>\d+)/$', 'django_nyt.views.goto', name='goto'),
    url('^goto/$', 'django_nyt.views.goto', name='goto_base'),)


def get_pattern(app_name="nyt", namespace="nyt"):
    """Every url resolution takes place as "nyt:view_name".
       https://docs.djangoproject.com/en/dev/topics/http/urls/#topics-http-reversing-url-namespaces
    """
    return urlpatterns, app_name, namespace



# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_nyt', '0002_notification_settings'),
    ]

    operations = [
        migrations.CreateModel(
            name='Subscription', fields=[
                ('id', models.AutoField(
                    auto_created=True, verbose_name='ID', serialize=False, primary_key=True)), ('settings', models.ForeignKey(
                    to_field='id', to='django_nyt.Settings')), ('notification_type', models.ForeignKey(
                    to_field='key', to='django_nyt.NotificationType')), ('object_id', models.CharField(
                    help_text='Leave this blank to subscribe to any kind of object', null=True, max_length=64, blank=True)), ('send_emails', models.BooleanField(
                    default=True)), ('latest', models.ForeignKey(
                    to_field='id', to='django_nyt.Notification', blank=True, null=True)), ], options={
                'verbose_name': 'subscription', 'db_table': 'nyt_subscription', 'verbose_name_plural': 'subscriptions', }, bases=(
                models.Model,), ), ]
